//---------------------------------------------------------------------------


#pragma hdrstop

#include "ControlQueue.h"
#include "Mutex.h"
#include "ScopedLock.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

#include "fifo.h"
#include "Command.h"

namespace
{
	Fifo<Command, 512> fifo;
	Mutex mutex;
}

ControlQueue::ControlQueue()
{

}

int ControlQueue::GetCommand(Command& cmd)
{
	ScopedLock<Mutex> lock(mutex);
	Command *tmpcmd = (Command*)fifo.getReadable();
	if (tmpcmd == NULL)
	{
		return 1;
	}
	cmd = *tmpcmd;
	fifo.pop();
	return 0;
}

void ControlQueue::Call(int accountId, AnsiString target)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	cmd->type = Command::CALL;
	cmd->accountId = accountId;
	cmd->target = target;
	fifo.push();
}

void ControlQueue::Answer(int callId)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	cmd->type = Command::ANSWER;
	cmd->callId = callId;
	fifo.push();
}

void ControlQueue::SendDigit(int callId, char key)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	cmd->type = Command::SEND_DIGIT;
	cmd->callId = callId;
	cmd->key = key;
	fifo.push();
}

void ControlQueue::Hangup(int callId)
{
	ScopedLock<Mutex> lock(mutex);
	Command *cmd = (Command*)fifo.getWriteable();
	if (!cmd)
		return;
	cmd->type = Command::HANGUP;
	cmd->callId = callId;
	fifo.push();
}


